import { RecordAuthResponse, RecordSubscription, UnsubscribeFunc } from 'pocketbase';
import { Collections, CompaniesRecord, CompaniesResponse } from '../types/pocketbase.types';
import { PocketbaseService } from './pocketbase.service';
import { useEventBus } from '@vueuse/core';
import { AuthService } from './auth.service';

const API = PocketbaseService.getApi();
const COLLECTION = API.collection(Collections.Companies);

export class CompanyService {
  public static async addCompany(company: CompaniesRecord & {password: string}): Promise<CompaniesResponse> {
    return COLLECTION.create({ ...company, passwordConfirm: company.password }, { $cancelKey: `${company.name}` });
  }
  public static async getCompanies(): Promise<CompaniesResponse[]> {
    return COLLECTION.getFullList({ sort: 'name' });
  }
  public static async login(username: string, password: string): Promise<RecordAuthResponse<CompaniesResponse>> {
    try {
      const response = await COLLECTION.authWithPassword<CompaniesResponse>(username, password);
      useEventBus<boolean>('isAdmin').emit(AuthService.isAdmin());
      return response;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    } catch (error: any) {
      throw new Error(error);
    }
  }
  public static isAuthenticated(): boolean {
    if(API.authStore.isValid && API.authStore.model) {
      return Boolean(API.authStore.model.collectionId);
    }
    this.logout();
    return false;
  }
  public static logout(): void {
    API.authStore.clear();
  }
  public static async getCompany(): Promise<CompaniesResponse> {
    if(this.isAuthenticated()) {
      return API.authStore.model as unknown as CompaniesResponse;
    }
    throw new Error('Not authenticated');
  }
  public static async getCompanyByAccountNumber(accountNumber: string): Promise<CompaniesResponse> {
    return COLLECTION.getFirstListItem(`accountNumber="${accountNumber}"`);
  }
  public static async updateAccountNumber(accountId: string, accountNumber: string): Promise<CompaniesResponse> {
    return COLLECTION.update(accountId, { accountNumber });
  }
  public static async subscribeToCompanyChanges(
    callback: (data: RecordSubscription<CompaniesResponse>)=> void,
  ): Promise<UnsubscribeFunc> {
    return await COLLECTION.subscribe<CompaniesResponse>('*', (data) => {
      callback(data);
    });
  }
}